
#include <math.h>
#include <stdio.h>

int Nrand;
double GaussAdd, GaussFac, Arand;

double **malloc_matrix(int,int,int,int);
void free_matrix(double**,int,int,int,int);

#define F3(delta,x0,x1,x2)      ((x0+x1+x2)/3 + delta*Gauss())
#define F4(delta,x0,x1,x2,x3)   ((x0+x1+x2+x3)/4 + delta*Gauss())

void InitGauss( seed )
int seed;
{
    Nrand = 4;
    Arand = pow( 2.0, 31.0 ) - 1.;
    GaussAdd = sqrt( 3.0 * Nrand );
    GaussFac = 2 * GaussAdd / ( Nrand * Arand );
    srand( seed );
}


double Gauss( )
{
    double sum;
    int i;

    sum = 0;
    for( i=1; i<=Nrand; i++ )
        sum += sum + rand();
    return( GaussFac * sum - GaussAdd );
}


#if 0
main()
{
    int seed, maxlevel, addition;
    double H, sigma;

    seed = time(0);
    maxlevel = 4;
    H = 0.8;
    sigma = 0.8;
    addition = 1;

    MidPointFM2D( maxlevel, seed, addition, H, sigma );
}
#endif

MidPointFM2D( maxlevel, seed, addition, H, sigma )
int maxlevel, seed, addition;
double H, sigma;
{
    FILE *fp;
    extern  char *filename;

    double **X;
    int i, stage, x, y, N, D, d;
    double delta;
    double min=65536., max=-65536.;

    InitGauss( seed );

    N = (int)pow( 2.0, (double)maxlevel );
    delta = sigma;

    X = malloc_matrix(0,N,0,N);

    X[0][0] = delta * Gauss();
    X[0][N] = delta * Gauss();
    X[N][0] = delta * Gauss();
    X[N][N] = delta * Gauss();

    D = N;
    d = (int)(N / 2.0);

    for( stage = 1; stage <= maxlevel; stage ++ ) {

        delta *= pow( 0.5, 0.5*H );

        for( x=d; x<=N-d; x+=D )
            for( y=d; y<=N-d; y+=D )
                X[x][y] = F4( delta, X[x+d][y+d], X[x+d][y-d], X[x-d][y+d], X[x-d][y-d]);
        if( addition )
            for( x=d; x<=N-d; x+=D )
                for( y=d; y<=N-d; y+=D )
                    X[x][y] += delta * Gauss();

        delta *= pow( 0.5, 0.5*H );

        for( x=d; x<=N-d; x+=D ) {
            X[x][0] = F3( delta, X[x+d][0], X[x-d][0], X[x][d] );
            X[x][N] = F3( delta, X[x+d][N], X[x-d][N], X[x][N-d] );
            X[0][x] = F3( delta, X[0][x+d], X[0][x-d], X[d][x] );
            X[N][x] = F3( delta, X[N][x+d], X[N][x-d], X[N-d][x] );
        }
        for( x=d; x<=N-d; x+=D )
            for( y=D; y<=N-d; y+=D )
                X[x][y] = F4( delta, X[x][y+d], X[x][y-d], X[x+d][y], X[x-d][y] );
        for( x=D; x<=N-d; x+=D )
            for( y=d; y<=N-d; y+=D )
                X[x][y] = F4( delta, X[x][y+d], X[x][y-d], X[x+d][y], X[x-d][y] );
        if( addition ) {
            for( x=0; x<=N; x+=D )
                for( y=0; y<=N; y+=D )
                    X[x][y] += delta * Gauss();
            for( x=d; x<=N-d; x+=D )
                for( y=d; y<=N-d; y+=D )
                    X[x][y] += delta * Gauss();
        }
        D = (int)(D / 2.0);
        d = (int)(d / 2.0);
    }

    for (y=0; y<=N; y++)
    for (x=0; x<=N; x++)
        if( X[x][y] > max )
            max = X[x][y];
        else if( X[x][y] < min )
            min = X[x][y];
    if( max > 0.0 )
        for (y=0; y<=N; y++)
        for (x=0; x<=N; x++) {
            if( X[x][y] < (min+0.05*(max-min)) )
                X[x][y] = min+0.05*(max-min);
            X[x][y] /= max;
        }
        
    /*
     * Print out the points into the heightfield file
     */
    fp = fopen( filename, "w" );
    fprintf( fp,"%d\t%d\t%d\n", (N+1)*(N+1), N*N, 4*N*N );
    i = 0;
    for (y=0; y<=N; y++)
        for (x=0; x<=N; x++) {
            fprintf( fp,"%6.3f\t%6.3f\t%6.3f\n",
                     ((float)(i%(N+1))/(float)((N+1)-1))-0.5,
                     X[x][y]-0.5,
                     ((float)(i/(N+1))/(float)((N+1)-1))-0.5 );
            i++;
        }

    i = 0;
    for (y=0; y<N; y++)
        for (x=0; x<N; x++) {
                fprintf( fp,"4\t%d\t%d\t%d\t%d\n",
                           x*(N+1)+y+1, (x+1)*(N+1)+y+1,
                   (x+1)*(N+1)+(y+1)+1, x*(N+1)+(y+1)+1 );
                i++;
        }
    fclose(fp);

    free_matrix(X,0,N,0,N);
}


/*
 * Allocate memory for a matrix
 */

double **malloc_matrix(nrl,nrh,ncl,nch)
int nrl,nrh,ncl,nch;
{
    int i;
    double **m;

    m=(double **) malloc((unsigned) (nrh-nrl+1)*sizeof(double*));
    if (!m) {
        fprintf( stderr, "allocation failure 1 in matrix() -- %d %d %d %d\n", nrl,nrh,ncl,nch );
        exit(0);
    }
    m -= nrl;

    for(i=nrl;i<=nrh;i++) {
        m[i]=(double *) malloc((unsigned) (nch-ncl+1)*sizeof(double));
        if (!m[i]) {
            fprintf( stderr, "allocation failure 2 in matrix() -- %d %d %d %d\n", nrl,nrh,ncl,nch );
            exit(0);
        }
        m[i] -= ncl;
    }
    return m;
}

/*
 * Free memory used by a matrix
 */

void free_matrix(m,nrl,nrh,ncl,nch)
double **m;
int nrl,nrh,ncl,nch;
{
    int i;

    for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
    free((char*) (m+nrl));
}
