
#include "x.h"
#include "xt.h"
#include "xpex.h"
#include <math.h>
#include <stdlib.h>
#include <strings.h>
#include "book_utils.h"
#include "externs.h"
#include "defs.h"
#include "globals.h"

extern Display *dpy;
extern Window   window, windowWindow;

extern PEXRenderer           renderer;
extern PEXRendererAttributes rattrs;
extern PEXViewEntry          view;

extern PEXStructure top_struct, view_struct;
Atom                delete_window_atom;
extern Widget       toplevel, canvas;

int old_x, old_y, new_x, new_y;

/* Callback function to quit the program. */
void
quit( w, client_data, call_data )
Widget     w;
caddr_t    client_data;
XtPointer *call_data;
{
#if 0
    XClientMessageEvent        *cm = &call_data->event->xclient;

    if ( cm->data.l[0] == delete_window_atom ) {
        XtDestroyWidget( toplevel );
        exit( 0 );
    }
#endif
}


/* Callback for Expose events. */
void
exposeWindow( w, client_data, event )
Widget    w;
XtPointer    client_data;
XEvent    *event;
{
    /* Redraw when last Expose event received. */
    if ( event->xexpose.count == 0 )
        redrawWindow();
}


/*
 * ( 0, 1)       ( 1, 1)
 * (t1,t4)       (t3,t4)
 *       ---------
 *       |       |
 *       |       |
 *       |       |
 *       |       |
 *       ---------
 * (t1,t2)       (t3,t2)
 * ( 0, 0)       ( 1, 0)
 *
 */

void
pressWindow( w, client_data, event )
Widget    w;
XtPointer    client_data;
XEvent    *event;
{
    extern Widget viewWindowLLLabel, viewWindowURLabel;

    XWindowAttributes xwattrs;
    double dist00, dist01, dist10, dist11;
    int width, height;
    int error;
    int t1, t2, t3, t4;
    String string = (String) malloc(30);

    if( event->xbutton.button == 1 ) {
        XGetWindowAttributes( dpy, windowWindow, &xwattrs );
        width = xwattrs.width;
        height = xwattrs.height;
        XSetForeground( dpy, gc, 1 );
        XFillRectangle( dpy, windowWindow, gc, 0, 0, width, height );
        XSetForeground( dpy, gc, 0 );

        t1 =       (int)(((float)width/2.)+view_window[0].x*(float)width / 10.);
        t2 = width-(int)(((float)width/2.)+view_window[0].y*(float)width / 10.);
        t3 =       (int)(((float)width/2.)+view_window[1].x*(float)width / 10.);
        t4 = width-(int)(((float)width/2.)+view_window[1].y*(float)width / 10.);
        new_x = event->xbutton.x;
        new_y = event->xbutton.y;
        dist00 = sqrt( (new_y-t2)*(new_y-t2) + (new_x-t1)*(new_x-t1) );
        dist01 = sqrt( (new_y-t4)*(new_y-t4) + (new_x-t1)*(new_x-t1) );
        dist10 = sqrt( (new_y-t2)*(new_y-t2) + (new_x-t3)*(new_x-t3) );
        dist11 = sqrt( (new_y-t4)*(new_y-t4) + (new_x-t3)*(new_x-t3) );

        if( (dist00 < dist01) && (dist00 < dist10) && (dist00 < dist11) ) {
            view_window[0].x = -5. + (float)new_x         * 10. / (float)width;
            view_window[0].y = -5. + (float)(width-new_y) * 10. / (float)width;
        }
        else if( (dist01 < dist00) && (dist01 < dist10) && (dist01 < dist11) ) {
            view_window[0].x = -5. + (float)new_x         * 10. / (float)width;
            view_window[1].y = -5. + (float)(width-new_y) * 10. / (float)width;
        }
        else if( (dist10 < dist00) && (dist10 < dist01) && (dist10 < dist11) ) {
            view_window[0].y = -5. + (float)(width-new_y) * 10. / (float)width;
            view_window[1].x = -5. + (float)new_x         * 10. / (float)width;
        }
        else if( (dist11 < dist00) && (dist11 < dist01) && (dist11 < dist10) ) {
            view_window[1].x = -5. + (float)new_x         * 10. / (float)width;
            view_window[1].y = -5. + (float)(width-new_y) * 10. / (float)width;
        }

        sprintf( string, "%.2f,%.2f", view_window[0].x, view_window[0].y );
        XtVaSetValues( viewWindowLLLabel, XtNlabel, string, NULL );
        sprintf( string, "%.2f,%.2f", view_window[1].x, view_window[1].y );
        XtVaSetValues( viewWindowURLabel, XtNlabel, string, NULL );

        /* Compute the view orientation matrix. */
        error = PEXViewMappingMatrix( view_window, &viewport, perspective,
            &proj_ref_pt, view_plane, back_plane,
            front_plane, view.mapping );
        PEXSetTableEntries( dpy, rattrs.view_table, 1, 1, PEXLUTView, &view );
        redrawWindow();
        redraw();
    }
}


void
motionWindow( w, client_data, event )
Widget    w;
XtPointer    client_data;
XEvent    *event;
{
    if( event->xbutton.button == 1 ) {
    }
}


void
releaseWindow( w, client_data, event )
Widget    w;
XtPointer    client_data;
XEvent    *event;
{
    if( event->xbutton.button == 1 ) {
    }
}


/* Callback for Expose events. */
void
expose( w, client_data, event )
Widget    w;
XtPointer    client_data;
XEvent    *event;
{
    /* Redraw when last Expose event received. */
    if ( event->xexpose.count == 0 )
        redraw();
}


update_view_structure()
{
    PEXCoord *points_vpn, tmp_vpn;
    PEXCoord *points_vup, tmp_vup;

    PEXSetEditingMode( dpy, view_struct, PEXStructureReplace );
    PEXSetElementPtr( dpy, view_struct, PEXBeginning, 0 );

    PEXSetElementPtrAtLabel( dpy, view_struct, VRP_LABEL, 1 );
    PEXMarkers( dpy, view_struct, PEXOCStore, 1, &view_ref_pt );

    points_vup = (PEXCoord *)malloc( 2 * sizeof(PEXCoord) );
    points_vup[0] = view_ref_pt;
    tmp_vup.x = view_up_vec.x;
    tmp_vup.y = view_up_vec.y;
    tmp_vup.z = view_up_vec.z;
    points_vup[1] = tmp_vup;
    PEXSetElementPtrAtLabel( dpy, view_struct, VUP_LABEL, 1 );
    PEXPolyline( dpy, view_struct, PEXOCStore, 2, points_vup );

    points_vpn = (PEXCoord *)malloc( 2 * sizeof(PEXCoord) );
    points_vpn[0] = view_ref_pt;
    tmp_vpn.x = view_plane_normal.x;
    tmp_vpn.y = view_plane_normal.y;
    tmp_vpn.z = view_plane_normal.z;
    points_vpn[1] = tmp_vpn;
    PEXSetElementPtrAtLabel( dpy, view_struct, VPN_LABEL, 1 );
    PEXPolyline( dpy, view_struct, PEXOCStore, 2, points_vpn );
}


void
set_vrp( w, client_data, cbs )
Widget    w;
XtPointer client_data;
XtPointer cbs;
{
    int error;

    extern Widget viewVRPxScroll, viewVRPyScroll, viewVRPzScroll, viewVRPLabel;
    extern PEXCoord view_ref_pt;
    String string = (String) malloc(30);
    float per = *( float* )cbs;
    float top = ( float )per;

    switch ( (int)client_data ) {
    case VRP_X:
        view_ref_pt.x = -10. + top * 20.;
        break;
    case VRP_Y:
        view_ref_pt.y = -10. + top * 20.;
        break;
    case VRP_Z:
        view_ref_pt.z = -10. + top * 20.;
        break;
    }

    sprintf( string, "(%.2f,%.2f,%.2f)",
        view_ref_pt.x, view_ref_pt.y, view_ref_pt.z );
    XtVaSetValues( viewVRPLabel, XtNlabel, string, NULL );

    /* Compute the view orientation matrix. */
    error = PEXViewOrientationMatrix( &view_ref_pt, &view_plane_normal,
        &view_up_vec, view.orientation );
    PEXSetTableEntries( dpy, rattrs.view_table, 1, 1, PEXLUTView, &view );
    update_view_structure();
    redraw();
}


void
reset_vrp( w, client_data, cbs )
Widget    w;
XtPointer client_data;
XtPointer cbs;
{
    int error;

    extern Widget viewVRPxScroll, viewVRPyScroll, viewVRPzScroll, viewVRPLabel;
    extern PEXCoord view_ref_pt;
    String string = (String) malloc(30);
    float topOfThumb;
    XtArgVal *l_topOfThumb;

    view_ref_pt.x = 0.5;
    view_ref_pt.y = 0.5;
    view_ref_pt.z = 0.5;

    topOfThumb = 0.5 + view_ref_pt.x / 20.;
    l_topOfThumb = (XtArgVal *) &topOfThumb;
    XtVaSetValues( viewVRPxScroll, XtNtopOfThumb, *l_topOfThumb, NULL );
    topOfThumb = 0.5 + view_ref_pt.y / 20.;
    l_topOfThumb = (XtArgVal *) &topOfThumb;
    XtVaSetValues( viewVRPyScroll, XtNtopOfThumb, *l_topOfThumb, NULL );
    topOfThumb = 0.5 + view_ref_pt.z / 20.;
    l_topOfThumb = (XtArgVal *) &topOfThumb;
    XtVaSetValues( viewVRPzScroll, XtNtopOfThumb, *l_topOfThumb, NULL );

    sprintf( string, "(%.2f,%.2f,%.2f)",
        view_ref_pt.x, view_ref_pt.y, view_ref_pt.z );
    XtVaSetValues( viewVRPLabel, XtNlabel, string, NULL );

    /* Compute the view orientation matrix. */
    error = PEXViewOrientationMatrix( &view_ref_pt, &view_plane_normal,
        &view_up_vec, view.orientation );
    PEXSetTableEntries( dpy, rattrs.view_table, 1, 1, PEXLUTView, &view );
    update_view_structure();
    redraw();
}


void
set_vup( w, client_data, cbs )
Widget    w;
XtPointer client_data;
XtPointer cbs;
{
    int error;

    extern Widget viewVUPxScroll, viewVUPyScroll, viewVUPzScroll, viewVUPLabel;
    extern PEXVector view_up_vec;
    String string = (String) malloc(30);
    float per = *( float* )cbs;
    float top = ( float )per;

    switch ( (int)client_data ) {
    case VUP_X:
        view_up_vec.x = -10. + top * 20.;
        break;
    case VUP_Y:
        view_up_vec.y = -10. + top * 20.;
        break;
    case VUP_Z:
        view_up_vec.z = -10. + top * 20.;
        break;
    }

    sprintf( string, "(%.2f,%.2f,%.2f)",
        view_up_vec.x, view_up_vec.y, view_up_vec.z );
    XtVaSetValues( viewVUPLabel, XtNlabel, string, NULL );

    /* Compute the view orientation matrix. */
    error = PEXViewOrientationMatrix( &view_ref_pt, &view_plane_normal,
        &view_up_vec, view.orientation );
    PEXSetTableEntries( dpy, rattrs.view_table, 1, 1, PEXLUTView, &view );
    update_view_structure();
    redraw();
}


void
reset_vup( w, client_data, cbs )
Widget    w;
XtPointer client_data;
XtPointer cbs;
{
    int error;

    extern Widget viewVUPxScroll, viewVUPyScroll, viewVUPzScroll, viewVUPLabel;
    extern PEXVector view_up_vec;
    String string = (String) malloc(30);
    float topOfThumb;
    XtArgVal *l_topOfThumb;

    view_up_vec.x = 0.0;
    view_up_vec.y = 1.0;
    view_up_vec.z = 0.0;

    topOfThumb = 0.5 + view_up_vec.x / 20.;
    l_topOfThumb = (XtArgVal *) &topOfThumb;
    XtVaSetValues( viewVUPxScroll, XtNtopOfThumb, *l_topOfThumb, NULL );
    topOfThumb = 0.5 + view_up_vec.y / 20.;
    l_topOfThumb = (XtArgVal *) &topOfThumb;
    XtVaSetValues( viewVUPyScroll, XtNtopOfThumb, *l_topOfThumb, NULL );
    topOfThumb = 0.5 + view_up_vec.z / 20.;
    l_topOfThumb = (XtArgVal *) &topOfThumb;
    XtVaSetValues( viewVUPzScroll, XtNtopOfThumb, *l_topOfThumb, NULL );

    sprintf( string, "(%.2f,%.2f,%.2f)",
        view_up_vec.x, view_up_vec.y, view_up_vec.z );
    XtVaSetValues( viewVUPLabel, XtNlabel, string, NULL );

    /* Compute the view orientation matrix. */
    error = PEXViewOrientationMatrix( &view_ref_pt, &view_plane_normal,
        &view_up_vec, view.orientation );
    PEXSetTableEntries( dpy, rattrs.view_table, 1, 1, PEXLUTView, &view );
    update_view_structure();
    redraw();
}


void
set_vpn( w, client_data, cbs )
Widget    w;
XtPointer client_data;
XtPointer cbs;
{
    int error;

    extern Widget viewVPNzScroll, viewVPNyScroll, viewVPNzScroll, viewVPNLabel;
    extern PEXVector view_plane_normal;
    String string = (String) malloc(30);
    float per = *( float* )cbs;
    float top = ( float )per;

    switch ( (int)client_data ) {
    case VPN_X:
        view_plane_normal.x = -10. + top * 20.;
        break;
    case VPN_Y:
        view_plane_normal.y = -10. + top * 20.;
        break;
    case VPN_Z:
        view_plane_normal.z = -10. + top * 20.;
        break;
    }

    sprintf( string, "(%.2f,%.2f,%.2f)",
        view_plane_normal.x, view_plane_normal.y, view_plane_normal.z );
    XtVaSetValues( viewVPNLabel, XtNlabel, string, NULL );

    /* Compute the view orientation matrix. */
    error = PEXViewOrientationMatrix( &view_ref_pt, &view_plane_normal,
        &view_up_vec, view.orientation );
    PEXSetTableEntries( dpy, rattrs.view_table, 1, 1, PEXLUTView, &view );
    update_view_structure();
    redraw();
}


void
reset_vpn( w, client_data, cbs )
Widget    w;
XtPointer client_data;
XtPointer cbs;
{
    int error;

    extern Widget viewVPNxScroll, viewVPNyScroll, viewVPNzScroll, viewVPNLabel;
    extern PEXCoord view_ref_pt;
    String string = (String) malloc(30);
    float topOfThumb;
    XtArgVal *l_topOfThumb;

    view_plane_normal.x = 1.0;
    view_plane_normal.y = 1.0;
    view_plane_normal.z = 1.0;

    topOfThumb = 0.5 + view_plane_normal.x / 20.;
    l_topOfThumb = (XtArgVal *) &topOfThumb;
    XtVaSetValues( viewVPNxScroll, XtNtopOfThumb, *l_topOfThumb, NULL );
    topOfThumb = 0.5 + view_plane_normal.y / 20.;
    l_topOfThumb = (XtArgVal *) &topOfThumb;
    XtVaSetValues( viewVPNyScroll, XtNtopOfThumb, *l_topOfThumb, NULL );
    topOfThumb = 0.5 + view_plane_normal.z / 20.;
    l_topOfThumb = (XtArgVal *) &topOfThumb;
    XtVaSetValues( viewVPNzScroll, XtNtopOfThumb, *l_topOfThumb, NULL );

    sprintf( string, "(%.2f,%.2f,%.2f)",
        view_plane_normal.x, view_plane_normal.y, view_plane_normal.z );
    XtVaSetValues( viewVPNLabel, XtNlabel, string, NULL );

    /* Compute the view orientation matrix. */
    error = PEXViewOrientationMatrix( &view_ref_pt, &view_plane_normal,
        &view_up_vec, view.orientation );
    PEXSetTableEntries( dpy, rattrs.view_table, 1, 1, PEXLUTView, &view );
    update_view_structure();
    redraw();
}


void
set_prp( w, client_data, cbs )
Widget    w;
XtPointer client_data;
XtPointer cbs;
{
    int error;

    extern Widget viewPRPxScroll, viewPRPyScroll, viewPRPzScroll, viewPRPLabel;
    extern PEXCoord proj_ref_pt;
    String string = (String) malloc(30);
    float per = *( float* )cbs;
    float top = ( float )per;

    switch ( (int)client_data ) {
    case PRP_X:
        proj_ref_pt.x = -10. + top * 20.;
        break;
    case PRP_Y:
        proj_ref_pt.y = -10. + top * 20.;
        break;
    case PRP_Z:
        proj_ref_pt.z = -10. + top * 20.;
        break;
    }

    sprintf( string, "(%.2f,%.2f,%.2f)",
        proj_ref_pt.x, proj_ref_pt.y, proj_ref_pt.z );
    XtVaSetValues( viewPRPLabel, XtNlabel, string, NULL );

    /* Compute the view orientation matrix. */
    error = PEXViewMappingMatrix( view_window, &viewport, perspective,
        &proj_ref_pt, view_plane, back_plane,
        front_plane, view.mapping );
    PEXSetTableEntries( dpy, rattrs.view_table, 1, 1, PEXLUTView, &view );
/*  update_view_structure(); */
    redraw();
}


void
reset_prp( w, client_data, cbs )
Widget    w;
XtPointer client_data;
XtPointer cbs;
{
    int error;

    extern Widget viewPRPxScroll, viewPRPyScroll, viewPRPzScroll, viewPRPLabel;
    extern PEXCoord proj_ref_pt;
    String string = (String) malloc(30);
    float topOfThumb;
    XtArgVal *l_topOfThumb;

    proj_ref_pt.x =  0.;
    proj_ref_pt.y =  0.;
    proj_ref_pt.z = 10.;

    topOfThumb = 0.5 + proj_ref_pt.x / 20.;
    l_topOfThumb = (XtArgVal *) &topOfThumb;
    XtVaSetValues( viewPRPxScroll, XtNtopOfThumb, *l_topOfThumb, NULL );
    topOfThumb = 0.5 + proj_ref_pt.y / 20.;
    l_topOfThumb = (XtArgVal *) &topOfThumb;
    XtVaSetValues( viewPRPyScroll, XtNtopOfThumb, *l_topOfThumb, NULL );
    topOfThumb = 0.5 + proj_ref_pt.z / 20.;
    l_topOfThumb = (XtArgVal *) &topOfThumb;
    XtVaSetValues( viewPRPzScroll, XtNtopOfThumb, *l_topOfThumb, NULL );

    sprintf( string, "(%.2f,%.2f,%.2f)",
        proj_ref_pt.x, proj_ref_pt.y, proj_ref_pt.z );
    XtVaSetValues( viewPRPLabel, XtNlabel, string, NULL );

    /* Compute the view orientation matrix. */
    error = PEXViewMappingMatrix( view_window, &viewport, perspective,
        &proj_ref_pt, view_plane, back_plane,
        front_plane, view.mapping );
    PEXSetTableEntries( dpy, rattrs.view_table, 1, 1, PEXLUTView, &view );
/*  update_view_structure(); */
    redraw();
}


void
set_planes( w, client_data, cbs )
Widget    w;
XtPointer client_data;
XtPointer cbs;
{
    int error;

    extern Widget viewBackScroll, viewViewScroll, viewFrontScroll;
    extern Widget viewBackLabel, viewViewLabel, viewFrontLabel;
    extern double view_plane, front_plane, back_plane;
    String string = (String) malloc(30);
    float per = *( float* )cbs;
    float top = ( float )per;

    switch ( (int)client_data ) {
    case BACK:
        back_plane = -10. + top * 20.;
        sprintf( string, "(%.2f)", back_plane );
        XtVaSetValues( viewBackLabel, XtNlabel, string, NULL );
        break;
    case VIEW:
        view_plane = -10. + top * 20.;
        sprintf( string, "(%.2f)", view_plane );
        XtVaSetValues( viewViewLabel, XtNlabel, string, NULL );
        break;
    case FRONT:
        front_plane = -10. + top * 20.;
        sprintf( string, "(%.2f)", front_plane );
        XtVaSetValues( viewFrontLabel, XtNlabel, string, NULL );
        break;
    }

    /* Compute the view orientation matrix. */
    error = PEXViewMappingMatrix( view_window, &viewport, perspective,
        &proj_ref_pt, view_plane, back_plane,
        front_plane, view.mapping );
    PEXSetTableEntries( dpy, rattrs.view_table, 1, 1, PEXLUTView, &view );
/*  update_view_structure(); */
    redraw();
}


void
reset_planes( w, client_data, cbs )
Widget    w;
XtPointer client_data;
XtPointer cbs;
{
    int error;

    extern Widget viewBackScroll, viewViewScroll, viewFrontScroll;
    extern Widget viewBackLabel, viewViewLabel, viewFrontLabel;
    extern double view_plane, front_plane, back_plane;
    String string = (String) malloc(30);
    float topOfThumb;
    XtArgVal *l_topOfThumb;

    switch ( (int)client_data ) {
    case BACK:
        back_plane = -2.;
        sprintf( string, "(%.2f)", back_plane );
        XtVaSetValues( viewBackLabel, XtNlabel, string, NULL );
        topOfThumb = 0.5 + back_plane / 20.;
        l_topOfThumb = (XtArgVal *) &topOfThumb;
        XtVaSetValues( viewBackScroll, XtNtopOfThumb, *l_topOfThumb, NULL );
        break;
    case VIEW:
        view_plane = 0.;
        sprintf( string, "(%.2f)", view_plane );
        XtVaSetValues( viewViewLabel, XtNlabel, string, NULL );
        topOfThumb = 0.5 + view_plane / 20.;
        l_topOfThumb = (XtArgVal *) &topOfThumb;
        XtVaSetValues( viewViewScroll, XtNtopOfThumb, *l_topOfThumb, NULL );
        break;
    case FRONT:
        front_plane = 2.;
        sprintf( string, "(%.2f)", front_plane );
        XtVaSetValues( viewFrontLabel, XtNlabel, string, NULL );
        topOfThumb = 0.5 + front_plane / 20.;
        l_topOfThumb = (XtArgVal *) &topOfThumb;
        XtVaSetValues( viewFrontScroll, XtNtopOfThumb, *l_topOfThumb, NULL );
        break;
    }

    /* Compute the view orientation matrix. */
    error = PEXViewMappingMatrix( view_window, &viewport, perspective,
        &proj_ref_pt, view_plane, back_plane,
        front_plane, view.mapping );
    PEXSetTableEntries( dpy, rattrs.view_table, 1, 1, PEXLUTView, &view );
/*  update_view_structure(); */
    redraw();
}


void
reset_window( w, client_data, cbs )
Widget    w;
XtPointer client_data;
XtPointer cbs;
{
    extern Widget viewWindowLLLabel, viewWindowURLabel;

    int error;
    XWindowAttributes xwattrs;
    String string = (String) malloc(30);

    XGetWindowAttributes( dpy, windowWindow, &xwattrs );
    XSetForeground( dpy, gc, 1 );
    XFillRectangle( dpy, windowWindow, gc, 0,0, xwattrs.width,xwattrs.height );
    XSetForeground( dpy, gc, 0 );

    view_window[0].x = -1.5;
    view_window[0].y = -1.5;
    view_window[1].x =  1.5;
    view_window[1].y =  1.5;

    sprintf( string, "%.2f,%.2f", view_window[0].x, view_window[0].y );
    XtVaSetValues( viewWindowLLLabel, XtNlabel, string, NULL );
    sprintf( string, "%.2f,%.2f", view_window[1].x, view_window[1].y );
    XtVaSetValues( viewWindowURLabel, XtNlabel, string, NULL );

    /* Compute the view orientation matrix. */
    error = PEXViewMappingMatrix( view_window, &viewport, perspective,
        &proj_ref_pt, view_plane, back_plane,
        front_plane, view.mapping );
    PEXSetTableEntries( dpy, rattrs.view_table, 1, 1, PEXLUTView, &view );
/*  update_view_structure(); */
    redrawWindow();
    redraw();
}


void
set_type( w, client_data, cbs )
Widget    w;
XtPointer client_data;
XtPointer cbs;
{
    int error;
    extern Widget viewTypeLabel, viewTypeButton, viewTypeEntry[2];
    extern Pixmap mark;
    extern int perspective;

    switch ( (int)client_data ) {
    case 0:
        if( !perspective ) {
            perspective = True;
            XtVaSetValues( viewTypeEntry[0], XtNleftBitmap, mark, NULL );
            XtVaSetValues( viewTypeEntry[1], XtNleftBitmap, None, NULL );
            XtVaSetValues( viewTypeButton, XtNlabel, "Perspective", NULL );
        }
        break;
    case 1:
        if( perspective ) {
            perspective = False;
            XtVaSetValues( viewTypeEntry[0], XtNleftBitmap, None, NULL );
            XtVaSetValues( viewTypeEntry[1], XtNleftBitmap, mark, NULL );
            XtVaSetValues( viewTypeButton, XtNlabel, "Parallel", NULL );
        }
        break;
    }

    /* Compute the view orientation matrix. */
    error = PEXViewMappingMatrix( view_window, &viewport, perspective,
        &proj_ref_pt, view_plane, back_plane,
        front_plane, view.mapping );
    PEXSetTableEntries( dpy, rattrs.view_table, 1, 1, PEXLUTView, &view );
/*  update_view_structure(); */
    redraw();
}


void
set_clip( w, client_data, cbs )
Widget    w;
XtPointer client_data;
XtPointer cbs;
{
    int error;
    extern Widget viewClipLabel, viewClipButton, viewClipEntry[2];
    extern Pixmap mark;

    switch ( (int)client_data ) {
    case 0:
        if( view.clip_flags != PEXClipBack ) {
            view.clip_flags =  PEXClipBack;
            XtVaSetValues( viewClipEntry[0], XtNleftBitmap, mark, NULL );
            XtVaSetValues( viewClipEntry[1], XtNleftBitmap, None, NULL );
            XtVaSetValues( viewClipEntry[2], XtNleftBitmap, None, NULL );
            XtVaSetValues( viewClipEntry[3], XtNleftBitmap, None, NULL );
            XtVaSetValues( viewClipButton, XtNlabel, "Back", NULL );
        }
        break;
    case 1:
        if( view.clip_flags != PEXClipFront ) {
            view.clip_flags =  PEXClipFront;
            XtVaSetValues( viewClipEntry[0], XtNleftBitmap, None, NULL );
            XtVaSetValues( viewClipEntry[1], XtNleftBitmap, mark, NULL );
            XtVaSetValues( viewClipEntry[2], XtNleftBitmap, None, NULL );
            XtVaSetValues( viewClipEntry[3], XtNleftBitmap, None, NULL );
            XtVaSetValues( viewClipButton, XtNlabel, "Front", NULL );
        }
        break;
    case 2:
        if( view.clip_flags != PEXClipXY ) {
            view.clip_flags =  PEXClipXY;
            XtVaSetValues( viewClipEntry[0], XtNleftBitmap, None, NULL );
            XtVaSetValues( viewClipEntry[1], XtNleftBitmap, None, NULL );
            XtVaSetValues( viewClipEntry[2], XtNleftBitmap, mark, NULL );
            XtVaSetValues( viewClipEntry[3], XtNleftBitmap, None, NULL );
            XtVaSetValues( viewClipButton, XtNlabel, "XY", NULL );
        }
        break;
    case 3:
        if( view.clip_flags != PEXClippingAll ) {
            view.clip_flags =  PEXClippingAll;
            XtVaSetValues( viewClipEntry[0], XtNleftBitmap, None, NULL );
            XtVaSetValues( viewClipEntry[1], XtNleftBitmap, None, NULL );
            XtVaSetValues( viewClipEntry[2], XtNleftBitmap, None, NULL );
            XtVaSetValues( viewClipEntry[3], XtNleftBitmap, mark, NULL );
            XtVaSetValues( viewClipButton, XtNlabel, "All", NULL );
        }
        break;
    }

    PEXSetTableEntries( dpy, rattrs.view_table, 1, 1, PEXLUTView, &view );
/*  update_view_structure(); */
    redraw();
}


void
set_npc( w, client_data, cbs )
Widget    w;
XtPointer client_data;
XtPointer cbs;
{
    int error;

    extern PEXNPCSubVolume viewport;

    float per = *( float* )cbs;
    float top = ( float )per;

    switch ( (int)client_data ) {
    case NPC_MIN_X:
        viewport.min.x = top;
        break;
    case NPC_MIN_Y:
        viewport.min.y = top;
        break;
    case NPC_MIN_Z:
        viewport.min.z = top;
        break;
    case NPC_MAX_X:
        viewport.max.x = top;
        break;
    case NPC_MAX_Y:
        viewport.max.y = top;
        break;
    case NPC_MAX_Z:
        viewport.max.z = top;
        break;
    }

    /* Compute the view orientation matrix. */
    error = PEXViewMappingMatrix( view_window, &viewport, perspective,
        &proj_ref_pt, view_plane, back_plane,
        front_plane, view.mapping );
    PEXSetTableEntries( dpy, rattrs.view_table, 1, 1, PEXLUTView, &view );
/*  update_view_structure(); */
    redraw();
}
